@@ -45,6 +45,7 @@
 #include <commoncpp/config.h>
 #include <commoncpp/export.h>
 #include <commoncpp/string.h>
+#include <commoncpp/thread.h>
 #include <commoncpp/exception.h>
 
 #if defined(CCXX_EXCEPTIONS)
@@ -52,8 +53,8 @@
 using namespace COMMONCPP_NAMESPACE;
 
 Exception::Exception(const String& what_arg) throw():
-  _what(what_arg)
-{ }
+_what(what_arg)
+{}
 
 Exception::~Exception() throw()
 {}
@@ -89,12 +90,16 @@ const char* IOException::getSystemErrorString() const throw()
     if ( !_systemErrorString )
         _systemErrorString = new char[errStrSize];
 #ifndef _MSWINDOWS_
-#if (_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600) && ! _GNU_SOURCE
+#ifdef  HAVE_STRERROR_R
     strerror_r(_systemError, _systemErrorString, errStrSize);
-    return _systemErrorString;
 #else
-    return strerror_r(_systemError, _systemErrorString, errStrSize);
+    static Mutex mlock;
+
+    mlock.enter();
+    String::set(_systemErrorString, errStrSize, strerror(_systemError));
+    mlock.leave();
 #endif
+    return _systemErrorString;
 #else
     FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM, NULL, _systemError,
         MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
