@@ -245,6 +245,33 @@ memalloc(size)
     members = 0;
     root = NULL;
     last = NULL;
+    index = NULL;
+}
+
+stringpager::stringpager(char **list, size_t size) :
+memalloc(size)
+{
+    members = 0;
+    root = NULL;
+    last = NULL;
+    add(list);
+}
+
+unsigned stringpager::token(const char *text, const char *list, const char *quote, const char *end)
+{
+    unsigned count = 0;
+    const char *result;
+    char *last = NULL;
+
+    if(!text || !*text)
+        return 0;
+
+    strdup_t tmp = strdup(text);
+    while(NULL != (result = String::token(tmp, &last, list, quote, end))) {
+        ++count;
+        add(result);
+    }
+    return count;
 }
 
 const char *stringpager::get(unsigned index)
@@ -266,6 +293,75 @@ void stringpager::clear(void)
     members = 0;
     root = NULL;
     last = NULL;
+    index = NULL;
+}
+
+const char *stringpager::pull(void)
+{
+    if(!members)
+        return NULL;
+
+
+    member *mem = (member *)root;
+    const char *result = mem->text;
+    --members;
+    if(!members) {
+        root = NULL;
+        last = NULL;
+    }
+    else
+        root = mem->next;
+    index = NULL;
+    return result;
+}
+
+void stringpager::push(const char *text)
+{
+    if(!text)
+        text = "";
+
+    size_t size = strlen(text) + 1;
+    caddr_t mem = (caddr_t)memalloc::_alloc(sizeof(member));
+    char *str = (char *)memalloc::_alloc(size);
+
+    strcpy(str, text);
+    member *node;
+
+    node = new(mem) member(&root, str);
+    if(!last)
+        last = node;
+    ++members;
+    index = NULL;
+}
+
+const char *stringpager::pop(void)
+{
+    const char *out = NULL;
+
+    if(!root)
+        return NULL;
+
+    index = NULL;
+
+    if(root == last) {
+        out = last->text;
+        root = last = NULL;
+        members = 0;
+        return out;
+    }
+
+    linked_pointer<member> np = root;
+    while(is(np)) {
+        if(np->next == last) {
+            out = last->text;
+            last = *np;
+            np->next = NULL;
+            --members;
+            break;
+        }
+        np.next();
+    }
+    return out;
 }
 
 void stringpager::add(const char *text)
@@ -280,6 +376,7 @@ void stringpager::add(const char *text)
     strcpy(str, text);
     member *node;
 
+    index = NULL;
     if(members++) {
         node = new(mem) member(str);
         last->set(node);
@@ -289,6 +386,24 @@ void stringpager::add(const char *text)
     last = node;
 }
 
+void stringpager::set(char **list)
+{
+    clear();
+    add(list);
+}
+
+void stringpager::push(char **list)
+{
+    const char *cp;
+    unsigned index = 0;
+
+    if(!list)
+        return;
+
+    while(NULL != (cp = list[index++]))
+        push(cp);
+}
+
 void stringpager::add(char **list)
 {
     const char *cp;
@@ -307,20 +422,37 @@ void stringpager::sort(void)
         return;
 
     member **list = new member*[members];
-    unsigned index = 0;
+    unsigned pos = 0;
     linked_pointer<member> mp = root;
 
     while(is(mp)) {
-        list[index++] = *mp;
+        list[pos++] = *mp;
         mp.next();
     }
 
     qsort(static_cast<void *>(list), members, sizeof(member *), &ncompare);
     root = NULL;
-    while(index)
-        list[--index]->enlist(&root);
+    while(pos)
+        list[--pos]->enlist(&root);
 
     delete list;
+    index = NULL;
+}
+
+char **stringpager::list(void)
+{
+    if(index)
+        return index;
+
+    unsigned pos = 0;
+    index = (char **)memalloc::_alloc(sizeof(char *) * (members + 1));
+    linked_pointer<member> mp = root;
+    while(is(mp)) {
+        index[pos++] = (char *)mp->text;
+        mp.next();
+    }
+    index[pos] = NULL;
+    return index;
 }
 
 DirPager::DirPager() :
@@ -658,6 +790,262 @@ memalloc(ps)
     cpos = 0;
 }
 
+void bufpager::set(const char *text)
+{
+    reset();
+    add(text);
+}
+
+void bufpager::put(const char *text, size_t iosize)
+{
+    _lock();
+
+    while(text && (iosize--)) {
+        if(!last || last->used == last->size) {
+            cpage_t *next;
+
+            if(freelist) {
+                next = freelist;
+                freelist = next->next;
+            }
+            else {
+                next = (cpage_t *)memalloc::_alloc(sizeof(cpage_t));
+                if(!next) {
+                    _unlock();
+                    return;
+                }
+
+                page_t *p = page;
+                unsigned size = 0;
+
+                while(p) {
+                    size = pagesize - p->used;
+                    if(size)
+                        break;
+                    p = p->next;
+                }
+                if(!p)
+                    p = pager();
+
+                if(!p) {
+                    _unlock();
+                    return;
+                }
+
+                next->text = ((char *)(p)) + p->used;
+                next->used = 0;
+                next->size = size;
+                p->used = pagesize;
+            }
+
+            if(last)
+                last->next = next;
+
+            if(!first)
+                first = next;
+            last = next;
+        }
+
+        ++ccount;
+        last->text[last->used++] = *(text++);
+    }
+    _unlock();
+}
+
+char *bufpager::copy(size_t *iosize)
+{
+    *iosize = 0;
+    _lock();
+    if(!current || (current->next == NULL && cpos >= current->used)) {
+        _unlock();
+        return NULL;
+    }
+
+    if(cpos >= current->used) {
+        current = current->next;
+        cpos = 0l;
+    }
+
+    char *result = current->text + cpos;
+    *iosize = current->used - cpos;
+    cpos = 0l;
+    current = current->next;
+    return result;
+}
+
+char *bufpager::request(size_t *iosize)
+{
+    *iosize = 0;
+    _lock();
+    if(!last || last->used >= last->size) {
+        cpage_t *next;
+        if(freelist) {
+            next = freelist;
+            freelist = next->next;
+        }
+        else {
+            next = (cpage_t *)memalloc::_alloc(sizeof(cpage_t));
+            if(!next) {
+                _unlock();
+                return NULL;
+            }
+
+            page_t *p = page;
+            unsigned size = 0;
+
+            while(p) {
+                size = pagesize - p->used;
+                if(size)
+                    break;
+                p = p->next;
+            }
+            if(!p)
+                p = pager();
+
+            if(!p) {
+                _unlock();
+                return NULL;
+            }
+            next->text = ((char *)(p)) + p->used;
+            next->used = 0;
+            next->size = size;
+            p->used = pagesize;
+        }
+
+        if(last)
+            last->next = next;
+
+        if(!first)
+            first = next;
+        last = next;
+    }
+    *iosize = last->size - last->used;
+    return last->text + last->used;
+}
+
+void bufpager::commit(size_t iosize)
+{
+    last->used += iosize;
+    _unlock();
+}
+
+void bufpager::release(void)
+{
+    _unlock();
+}
+
+size_t bufpager::get(char *text, size_t iosize)
+{
+    _lock();
+    if(!ccount) {
+        _unlock();
+        return 0;
+    }
+
+    unsigned long index = 0;
+
+    while(index < iosize && current) {
+        if(cpos >= current->used) {
+            if(!current->next)
+                break;
+            current = current->next;
+            cpos = 0l;
+        }
+        text[index++] = current->text[cpos++];
+    }
+    if(index < iosize)
+        text[index] = 0;
+    _unlock();
+    return index;
+}
+
+void bufpager::add(const char *text)
+{
+    _lock();
+
+    while(text && *text) {
+        if(!last || last->used == last->size) {
+            cpage_t *next;
+
+            if(freelist) {
+                next = freelist;
+                freelist = next->next;
+            }
+            else {
+                next = (cpage_t *)memalloc::_alloc(sizeof(cpage_t));
+                if(!next) {
+                    _unlock();
+                    return;
+                }
+
+                page_t *p = page;
+                unsigned size = 0;
+
+                while(p) {
+                    size = pagesize - p->used;
+                    if(size)
+                        break;
+                    p = p->next;
+                }
+                if(!p)
+                    p = pager();
+
+                if(!p) {
+                    _unlock();
+                    return;
+                }
+
+                next->text = ((char *)(p)) + p->used;
+                next->used = 0;
+                next->size = size;
+                p->used = pagesize;
+            }
+
+            if(last)
+                last->next = next;
+
+            if(!first)
+                first = next;
+            last = next;
+        }
+
+        ++ccount;
+        last->text[last->used++] = *(text++);
+    }
+    _unlock();
+}
+
+char *bufpager::dup(void)
+{
+    _lock();
+    if(!ccount) {
+        _unlock();
+        return NULL;
+    }
+
+    char *text = (char *)malloc(ccount + 1l);
+    if(!text) {
+        _unlock();
+        return NULL;
+    }
+
+    unsigned long index = 0, pos = 0;
+    cpage_t *page = first;
+
+    while(index < ccount && page) {
+        if(pos >= page->used) {
+            if(!page->next)
+                break;
+            page = page->next;
+            pos = 0l;
+        }
+        text[index++] = page->text[pos++];
+    }
+    text[index] = 0;
+    _unlock();
+    return text;
+}
+
 int bufpager::_getch(void)
 {
     _lock();
