@@ -2122,3 +2122,72 @@ uint16_t string::crc16(uint8_t *binary, size_t size)
     return crc;
 }
 
+NAMESPACE_UCOMMON
+
+#ifdef  HAVE_STRCOLL
+
+bool sort_eq(char const *str1, char const *str2)
+{
+    return strcoll(str1, str2) == 0;
+}
+
+bool sort_ne(char const *str1, char const *str2)
+{
+    return strcoll(str1, str2) != 0;
+}
+
+bool sort_lt(char const *str1, char const *str2)
+{
+    return strcoll(str1, str2) < 0;
+}
+
+bool sort_gt(char const *str1, char const *str2)
+{
+    return strcoll(str1, str2) > 0;
+}
+
+bool sort_le(char const *str1, char const *str2)
+{
+    return strcoll(str1, str2) <= 0;
+}
+
+bool sort_ge(char const *str1, char const *str2)
+{
+    return strcoll(str1, str2) >= 0;
+}
+
+#else
+
+bool sort_eq(char const *str1, char const *str2)
+{
+    return strcmp(str1, str2) == 0;
+}
+
+bool sort_ne(char const *str1, char const *str2)
+{
+    return strcmp(str1, str2) != 0;
+}
+
+bool sort_lt(char const *str1, char const *str2)
+{
+    return strcmp(str1, str2) < 0;
+}
+
+bool sort_gt(char const *str1, char const *str2)
+{
+    return strcmp(str1, str2) > 0;
+}
+
+bool sort_le(char const *str1, char const *str2)
+{
+    return strcmp(str1, str2) <= 0;
+}
+
+bool sort_ge(char const *str1, char const *str2)
+{
+    return strcmp(str1, str2) >= 0;
+}
+
+#endif
+
+END_NAMESPACE
