@@ -43,6 +43,10 @@
 #include <ucommon/string.h>
 #endif
 
+#ifndef _UCOMMON_MEMORY_H_
+#include <ucommon/memory.h>
+#endif
+
 #ifndef _MSWINDOWS_
 #include <sys/stat.h>
 #else
@@ -606,6 +610,14 @@ public:
      */
     static int load(const char *path);
 
+    /**
+     * Execute a process and get exit code.
+     * @param path to execute.
+     * @param argv list.
+     * @return exit code.
+     */
+    static int exec(const char *path, char **argv);
+
     /**
      * Load a plugin into memory.
      * @param module for management.
