@@ -277,6 +277,8 @@ public:
      */
     stringpager(size_t pagesize = 256);
 
+    stringpager(char **list, size_t pagesize = 256);
+
     /**
      * Get the number of items in the pager string list.
      * @return number of items stored.
@@ -298,6 +300,30 @@ public:
      */
     void add(const char *text);
 
+    /**
+     * Add text to front of list.
+     * @param text to add.
+     */
+    void push(const char *text);
+
+    /**
+     * Add text list to front of list.
+     * @param text to add.
+     */
+    void push(char **text);
+
+    /**
+     * Remove element from front of list.  Does not release memory.
+     * @return text removed.
+     */
+    const char *pull(void);
+
+    /**
+     * Remove element from back of list.  Does not release memory.
+     * @return text removed.
+     */
+    const char *pop(void);
+
     /**
      * Add list to list.  This is a list of string pointers terminated with
      * NULL.
@@ -305,6 +331,13 @@ public:
      */
     void add(char **list);
 
+    /**
+     * Set list to list.  This is a list of string pointers terminated with
+     * NULL.
+     * @param list of text to set.
+     */
+    void set(char **list);
+
     /**
      * Purge all members and release pager member.  The list can then
      * be added to again.
@@ -347,8 +380,42 @@ public:
      */
     void sort(void);
 
+    /**
+     * Gather index list.
+     * @return index.
+     */
+    char **list(void);
+
+    /**
+     * Tokenize a string and add each token to the stringpager.
+     * @param text to tokenize.
+     * @param list of characters to use as token separators.
+     * @param quote pairs of characters for quoted text or NULL if not used.
+     * @param end of line marker characters or NULL if not used.
+     * @return number of tokens parsed.
+     */
+    unsigned token(const char *text, const char *list, const char *quote = NULL, const char *end = NULL);
+
+
+
+    inline operator bool()
+        {return members > 0;}
+
+    inline bool operator!()
+        {return !members;}
+
+    inline stringpager& operator=(char **list)
+        {set(list); return *this;}
+
+    inline const char *operator*()
+        {return pull();}
+
+    inline operator char **()
+        {return list();};
+
 private:
     member *last;
+    char **index;
 };
 
 /**
@@ -440,14 +507,69 @@ public:
      */
     void rewind(void);
 
+    /**
+     * Create an output string from buffer.
+     * @return output string allocated.
+     */
+    char *dup(void);
+
+    /**
+     * Set text of string buffer.
+     * @param text to set.
+     */
+    void set(const char *text);
+
+    /**
+     * Add text to string buffer.
+     * @param text to add.
+     */
+    void add(const char *text);
+
+    /**
+     * Get string from buffer.
+     * @param text to save into.
+     * @param size of buffer.
+     * @return count of characters copied.
+     */
+    size_t get(char *text, size_t size);
+
+    /**
+     * Put memory string into buffer including NULL byte.
+     * @param text to add.
+     * @param size of text to add.
+     */
+    void put(const char *text, size_t size);
+
     /**
      * Get total size.
      * @return number of characters in buffer.
      */
     inline unsigned long getUsed(void)
-        {return ccount;};
+        {return ccount;}
+
+    /**
+     * Convenience operator to get text.
+     * @return text string of buffer.
+     */
+    inline char *operator *()
+        {return dup();}
+
+    /**
+     * Convenience operator to add to pager.
+     * @param text to add to list.
+     */
+    inline bufpager& operator<<(const char *text)
+        {add(text); return *this;};
 
     bufpager(size_t page = 0);
+
+    char *request(size_t *iosize);
+
+    char *copy(size_t *iosize);
+
+    void release(void);
+
+    void commit(size_t size);
 };
 
 /**
@@ -933,6 +1055,12 @@ typedef stringpager::member stringlistitem_t;
  */
 typedef DirPager dirlist_t;
 
+inline const char *shift(stringlist_t& list)
+    {return list.pull();}
+
+inline void unshift(stringlist_t& list, const char *text)
+    {list.push(text);}
+
 
 END_NAMESPACE
 
