@@ -20,6 +20,8 @@
 #include <ucommon/export.h>
 #include <ucommon/secure.h>
 #include <errno.h>
+#include "md5.h"
+#include "sha1.h"
 
 #ifdef  _MSWINDOWS_
 #include <wincrypt.h>
