@@ -43,5 +43,14 @@ extern "C" int main()
     assert(eq(mylist[1], "100"));
     assert(eq(mylist[2], "300"));
 
+    const char *str = *mylist;
+    assert(eq(str, "050"));
+    assert(eq(mylist[0], "100"));
+
+    char **list = mylist;
+    assert(eq(list[0], "100"));
+    assert(eq(list[1], "300"));
+
+    assert(list[2] == NULL);
     return 0;
 }
